package grammar;

import text.*;
import java_cup.runtime.*;

import java.io.*;
import java.util.*;

%%

%public
%type		Symbol
%char

%{

	private int lineNumber = 1;
	public int lineNumber() { return lineNumber; }
	
	public Symbol token( int tokenType, String text ) {
		Print.error().debugln( 
			"Obtain token " + sym.terminal_name( tokenType ) + " \"" + text + "\"" );
		return new Symbol( tokenType, yychar, yychar + yytext().length(), text );
		}
	
	public Symbol token( int tokenType ) {
		return token( tokenType, yytext() );
		}
	
%}

%init{
	yybegin( NORMALSTATE );
%init}

binConst	=	0[bB][01]+
decConst	=	[0-9]+
hexConst	=	0[xX][0-9a-fA-F]+
ident		=	[A-Za-z][A-Za-z0-9]*
space		=	[\ \t]
newline		=	\r|\n|\r\n
octEsc		=	\\[0-3][0-7][0-7]
hexEsc		=	\\[xX][0-9a-fA-F][0-9a-fA-F]
charEsc		=	\\[ntbrfva\\\'\"]
schar		=	[^\'\"\\\r\n]|{octEsc}|{hexEsc}|{charEsc}
strConst	=	\"{schar}*\"
%state	NORMALSTATE, COMMENT1STATE, COMMENT2STATE, LEXERRORSTATE

%%
<NORMALSTATE> {

	{newline}	{ lineNumber++; }
	{space}		{ }
	
	"("			{ return token( sym.LEFT ); }
	")"			{ return token( sym.RIGHT ); }
	"["			{ return token( sym.LEFTSQ ); }
	"]"			{ return token( sym.RIGHTSQ ); }
	"{"			{ return token( sym.LEFTBRACE ); }
	"}"			{ return token( sym.RIGHTBRACE ); }
	
	";"			{ return token( sym.SEMICOLON ); }
	","			{ return token( sym.COMMA ); }
	"."			{ return token( sym.DOT ); }
	
	"@"			{ return token( sym.AT ); }
	"="			{ return token( sym.ASSIGN ); }
	"?"			{ return token( sym.QUEST ); }
	":"			{ return token( sym.COLON ); }
	"||"		{ return token( sym.OR ); }
	"&&"		{ return token( sym.AND ); }
	"<"			{ return token( sym.LT ); }
	">"			{ return token( sym.GT ); }
	"<="		{ return token( sym.LE ); }
	">="		{ return token( sym.GE ); }
	"=="		{ return token( sym.EQ ); }
	"!="		{ return token( sym.NE ); }
	"<<"		{ return token( sym.LEFTSHIFT ); }
	">>"		{ return token( sym.RIGHTSHIFT ); }
	"+"			{ return token( sym.PLUS ); }
	"-"			{ return token( sym.MINUS ); }
	"&"			{ return token( sym.AMPERSAND ); }
	"*"			{ return token( sym.TIMES ); }
	"/"			{ return token( sym.DIVIDE ); }
	"%"			{ return token( sym.MOD ); }
	"!"			{ return token( sym.NOT ); }

	include		{ return token( sym.INCLUDE ); }
	define		{ return token( sym.DEFINE ); }
	path		{ return token( sym.PATH ); }
	in			{ return token( sym.IN ); }
	out			{ return token( sym.OUT ); }
	component	{ return token( sym.COMPONENT ); }
	begin		{ return token( sym.BEGIN ); }
	end			{ return token( sym.END ); }
	if			{ return token( sym.IF ); }
	then		{ return token( sym.THEN ); }
	else		{ return token( sym.ELSE ); }
	elif		{ return token( sym.ELIF ); }
	for			{ return token( sym.FOR ); }
	from		{ return token( sym.FROM ); }
	upto		{ return token( sym.UPTO ); }
	do			{ return token( sym.DO ); }
	
	true		{ return token( sym.TRUE ); }
	false		{ return token( sym.FALSE ); }
	
	{binConst}	{ return token( sym.BINLITERAL ); }
	{decConst}	{ return token( sym.DECLITERAL ); }
	{hexConst}	{ return token( sym.HEXLITERAL ); }
	{strConst}	{ return token( sym.STRINGLITERAL ); }
	{ident}		{ return token( sym.IDENT ); }
	
	"//"		{ yybegin( COMMENT1STATE ); }
	"/*"		{ yybegin( COMMENT2STATE ); }
	
	.			{
					yybegin( LEXERRORSTATE ); 
					return token( sym.error ); 
				}
	}

<LEXERRORSTATE> {
	{newline}	{ lineNumber++; yybegin( NORMALSTATE ); return token( sym.error ); }
	.			{ }
	}

<COMMENT1STATE> {
	{newline}	{ lineNumber++; yybegin( NORMALSTATE ); }
	.			{ }
	}
	
<COMMENT2STATE> {
	"*/"		{ yybegin( NORMALSTATE ); }
	{newline}	{ lineNumber++; }
	.			{ }
	}

<<EOF>>			{ return token( sym.EOF ); }

